# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2020-2024, by Samuel Williams.

require_relative "language/generic"
require_relative "language/ruby"

module Decode
	# Language specific parsers and definitions.
	module Language
	end
end
