# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2025, by Samuel Williams.

require "rbs"
require_relative "rbs/wrapper"
require_relative "rbs/class"
require_relative "rbs/method"
require_relative "rbs/module"
require_relative "rbs/generator"

module Decode
	# RBS generation functionality for Ruby type signatures.
	module RBS
	end
end
