# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2020-2024, by Samuel Williams.

require_relative "decode/version"
require_relative "decode/index"

# @namespace
module Decode
end
